jest.mock('@angular/router');

import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, Router } from '@angular/router';
import { of } from 'rxjs';

import { IEPreguntaCerrada, EPreguntaCerrada } from '../e-pregunta-cerrada.model';
import { EPreguntaCerradaService } from '../service/e-pregunta-cerrada.service';

import { EPreguntaCerradaRoutingResolveService } from './e-pregunta-cerrada-routing-resolve.service';

describe('Service Tests', () => {
  describe('EPreguntaCerrada routing resolve service', () => {
    let mockRouter: Router;
    let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
    let routingResolveService: EPreguntaCerradaRoutingResolveService;
    let service: EPreguntaCerradaService;
    let resultEPreguntaCerrada: IEPreguntaCerrada | undefined;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        providers: [Router, ActivatedRouteSnapshot],
      });
      mockRouter = TestBed.inject(Router);
      mockActivatedRouteSnapshot = TestBed.inject(ActivatedRouteSnapshot);
      routingResolveService = TestBed.inject(EPreguntaCerradaRoutingResolveService);
      service = TestBed.inject(EPreguntaCerradaService);
      resultEPreguntaCerrada = undefined;
    });

    describe('resolve', () => {
      it('should return IEPreguntaCerrada returned by find', () => {
        // GIVEN
        service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultEPreguntaCerrada = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultEPreguntaCerrada).toEqual({ id: 123 });
      });

      it('should return new IEPreguntaCerrada if id is not provided', () => {
        // GIVEN
        service.find = jest.fn();
        mockActivatedRouteSnapshot.params = {};

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultEPreguntaCerrada = result;
        });

        // THEN
        expect(service.find).not.toBeCalled();
        expect(resultEPreguntaCerrada).toEqual(new EPreguntaCerrada());
      });

      it('should route to 404 page if data not found in server', () => {
        // GIVEN
        jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as EPreguntaCerrada })));
        mockActivatedRouteSnapshot.params = { id: 123 };

        // WHEN
        routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
          resultEPreguntaCerrada = result;
        });

        // THEN
        expect(service.find).toBeCalledWith(123);
        expect(resultEPreguntaCerrada).toEqual(undefined);
        expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
      });
    });
  });
});
